package com.oqpo.api.service.rtnmng;


import com.oqpo.api.entity.loanmng.LoanMgntBaseInfoEntity;
import com.oqpo.api.entity.oper.UserMngEntity;
import com.oqpo.api.entity.pomng.PchReqBaseInfoEntity;
import com.oqpo.api.entity.pomng.PchReqDtlPtclEntity;
import com.oqpo.api.entity.rtnmng.RtnReqBaseInfoEntity;
import com.oqpo.api.entity.rtnmng.RtnReqDtlPtclEntity;
import com.oqpo.api.entity.stinfo.BrandEntity;
import com.oqpo.api.entity.stockmng.WhsMgntBaseInfoEntity;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.exception.GlobalException;
import com.oqpo.api.mapper.loanmng.LoanStateMapper;
import com.oqpo.api.mapper.oper.UserMngMapper;
import com.oqpo.api.mapper.pomng.PchReqMapper;
import com.oqpo.api.mapper.rtnmng.RtnReqMapper;
import com.oqpo.api.mapper.stinfo.BrandMapper;
import com.oqpo.api.mapper.stockmng.WhsMngMapper;
import com.oqpo.api.service.CommonService;
import com.oqpo.api.util.DateUtil;
import com.oqpo.api.web.dto.request.GridRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.PchReqInitInfoRequest;
import com.oqpo.api.web.dto.request.pomng.pchReq.SavePchReqlRequest;
import com.oqpo.api.web.dto.request.rtnmng.SaveRtnReqlRequest;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.*;
import com.oqpo.api.web.dto.response.rtnmng.rtnReq.RtnReqInfoResponse;
import com.oqpo.api.web.dto.response.rtnmng.rtnReq.RtnReqInitInfoResponse;
import com.oqpo.api.web.dto.response.rtnmng.rtnReq.RtnReqSearchListResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class RtnReqService extends CommonService {
    @Autowired
    private RtnReqMapper rtnReqMapper;

    @Autowired
    private UserMngMapper userMngMapper;

    @Autowired
    private BrandMapper brandMapper;

    @Autowired
    private WhsMngMapper whsMngMapper;

    /* 반품요청  초기 정보 */

    public RtnReqInitInfoResponse selectRtnReqInitInfo(String userId, PchReqInitInfoRequest pchReqInitInfoRequest) throws Exception {

        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);
        BrandEntity brandEntity = new BrandEntity();
        List<WhsMgntBaseInfoEntity> whsMgntList = new ArrayList<WhsMgntBaseInfoEntity>();
        WhsMgntBaseInfoEntity whsEntity = new WhsMgntBaseInfoEntity();

        if ("30".equals(uEntity.getAuthTpCd())) {   // 가맹점
            // 브랜드 정보 request 로 받아서 브랜드정보 가져와 넣어줌
            brandEntity = brandMapper.selectBrandNmInfo(pchReqInitInfoRequest.getBrandId());
            uEntity.setBrandId(pchReqInitInfoRequest.getBrandId());
            uEntity.setBrandNm(brandEntity.getBrandNm());

        } else if ("40".equals(uEntity.getAuthTpCd())) {  // 브랜드
            if (!pchReqInitInfoRequest.getBrandId().equals(uEntity.getBrandId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
            }
        } else if ("50".equals(uEntity.getAuthTpCd())) {  // 매장
            if ("".equals(pchReqInitInfoRequest.getStoreId())) {
                throw new GlobalException(SystemMessageCode.ERR_NONSTOREID);
            }

            if (!pchReqInitInfoRequest.getBrandId().equals(uEntity.getBrandId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
            }

            if (!pchReqInitInfoRequest.getStoreId().equals(uEntity.getStoreId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTSTOREBELONG);
            }

            //납품창고 정보 가져오기  selectStoreWhsLocList
            whsMgntList = whsMngMapper.selectStoreWhsLocList(pchReqInitInfoRequest.getBrandId(), pchReqInitInfoRequest.getStoreId());

            if (whsMgntList.size() == 1) {
                whsEntity = whsMgntList.get(0);
            }

        } else {
            // 브랜드 정보 request 로 받아서 브랜드정보 가져와 넣어줌
            brandEntity = brandMapper.selectBrandNmInfo(pchReqInitInfoRequest.getBrandId());
            uEntity.setBrandId(pchReqInitInfoRequest.getBrandId());
            uEntity.setBrandNm(brandEntity.getBrandNm());
        }

        return RtnReqInitInfoResponse.toDTO(uEntity,whsEntity);

    }

    /* 반품요청 저장 */
    @Transactional
    public void saveRtnReqInfo(String userId, SaveRtnReqlRequest saveRtnReqlRequest) throws Exception {
        try {

            // 로그인자 권한이 50 이면 매장 아이디 필수로 넘어와야함
            UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);
            String userNm = uEntity.getUserNm();

            if ("50".equals(uEntity.getAuthTpCd())) {
                if ("".equals(saveRtnReqlRequest.getStoreId())) {
                    throw new GlobalException(SystemMessageCode.ERR_NONSTOREID);
                }

                if (!saveRtnReqlRequest.getBrandId().equals(uEntity.getBrandId())) {
                    throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
                }

                if (!saveRtnReqlRequest.getStoreId().equals(uEntity.getStoreId())) {
                    throw new GlobalException(SystemMessageCode.ERR_NOTSTOREBELONG);
                }
            }  else if ("40".equals(uEntity.getAuthTpCd())){
                if (!saveRtnReqlRequest.getBrandId().equals(uEntity.getBrandId())) {
                    throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
                }
            }


            RtnReqBaseInfoEntity entity = new RtnReqBaseInfoEntity();

            String rtnReqUnqNo = "";
            if ("C".toString().equals(saveRtnReqlRequest.getViewCd())) {
                // 신규번호 생성
                entity.setRtnReqUnqNo(fnGetDealNo(18,""));
            } else {
                entity.setRtnReqUnqNo(saveRtnReqlRequest.getRtnReqUnqNo());
            }

            entity.setBrandId(saveRtnReqlRequest.getBrandId());
            entity.setStoreId(saveRtnReqlRequest.getStoreId());
            entity.setRtnReqStCd(saveRtnReqlRequest.getRtnReqStCd());
            entity.setNote(saveRtnReqlRequest.getNote());
            entity.setColReqDt(saveRtnReqlRequest.getColReqDt().replaceAll("\\.",""));
            entity.setRtnWhsId(saveRtnReqlRequest.getRtnWhsId());
            entity.setRtnLocation(saveRtnReqlRequest.getRtnLocation());
            entity.setRtnReqMgrNm(userNm);
            entity.setRtnReqMgrTelNo(uEntity.getTelNo());
            entity.setWhsDvsn(saveRtnReqlRequest.getWhsDvsn());



            // 품목 삭제 처리
            List<RtnReqDtlPtclEntity> gridDeleteData = saveRtnReqlRequest.toCodeEntities(saveRtnReqlRequest.getGridDeleteData());
            if (gridDeleteData.size() > 0) {
                rtnReqMapper.deleteRtnReqDtlPtcl(entity.getRtnReqUnqNo(), gridDeleteData) ;
            }

            // 품목 수정 처리
            List<RtnReqDtlPtclEntity> gridUpdateData = saveRtnReqlRequest.toCodeEntities(saveRtnReqlRequest.getGridUpdateData());
            if (gridUpdateData.size() > 0) {
                rtnReqMapper.updateRtnReqDtlPtcl(userId,  gridUpdateData) ;
            }

            // 품목 등록 처리
            List<RtnReqDtlPtclEntity> gridInsertData = saveRtnReqlRequest.toCodeEntities(saveRtnReqlRequest.getGridInsertData());
            if (gridInsertData.size() > 0) {

                int maxDtlNo = rtnReqMapper.selectMaxRtnReqDtlNo(entity.getRtnReqUnqNo());

                for (int i = 0 ; i < gridInsertData.size() ; i++) {
                    maxDtlNo = maxDtlNo + 10 ;
                    gridInsertData.get(i).setRtnReqUnqNo(entity.getRtnReqUnqNo());
                    gridInsertData.get(i).setRtnReqDtlNo( Integer.toString(maxDtlNo));
                    if (gridInsertData.get(i).getStoreUnitUnqNo() == null || "".equals(gridInsertData.get(i).getStoreUnitUnqNo())  ) {
                        gridInsertData.get(i).setUnitGubun("B");
                    } else {
                        gridInsertData.get(i).setUnitGubun("S");
                    }

                }

                rtnReqMapper.insertRtnReqDtlPtcl(userId, gridInsertData) ;
            }

            RtnReqBaseInfoEntity sumData = rtnReqMapper.selectSumRtnAmt(entity.getRtnReqUnqNo());
            entity.setRtnReqAmt(sumData.getRtnReqAmt());
            entity.setRtnReqItemQty(sumData.getRtnReqItemQty());

            if ("C".toString().equals(saveRtnReqlRequest.getViewCd())) {
                rtnReqMapper.insertRtnReqBaseInfo(userId, entity);
            } else {
                rtnReqMapper.updateRtnReqBaseInfo(userId, entity);
            }

        } catch (GlobalException e) {
            e.getStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.getStackTrace();
            throw new RuntimeException();
        }
    }

    /* 반품요청 그리드 리스트 */
    public GridResponse selectRtnReqGridList(String userId, String sBrandId, String sStoreId, String fromDt, String toDt, String sWhsId, String sRtnReqStCd, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        // 로그인자 권한이 50 이면 매장 아이디 필수로 넘어와야함
        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);

        if ("50".equals(uEntity.getAuthTpCd())) {
            if ("".equals(sStoreId)) {
                throw new GlobalException(SystemMessageCode.ERR_NONSTOREID);
            }

            if (!sBrandId.equals(uEntity.getBrandId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
            }

            if (!sStoreId.equals(uEntity.getStoreId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTSTOREBELONG);
            }
        }  else if ("40".equals(uEntity.getAuthTpCd())){
            if (!sBrandId.equals(uEntity.getBrandId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
            }
        }


        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = rtnReqMapper.selectRtnReqGridCnt( sBrandId,sStoreId,  fromDt, toDt, sWhsId,sRtnReqStCd, sItemNm );
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<RtnReqBaseInfoEntity> entities = rtnReqMapper.selectRtnReqGridList( sBrandId, sStoreId, fromDt, toDt, sWhsId,sRtnReqStCd, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> RtnReqSearchListResponse.builder()
                        .viewCd("R")
                        .rtnReqUnqNo(m.getRtnReqUnqNo() == null ? "" : m.getRtnReqUnqNo())
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .storeId(m.getStoreId() == null ? "" : m.getStoreId())
                        .storeNm(m.getStroeNm() == null ? "" : m.getStroeNm())
                        .rtnReqStCd(m.getRtnReqStCd() == null ? "" : m.getRtnReqStCd())
                        .rtnReqStNm(m.getRtnReqStNm() == null ? "" : m.getRtnReqStNm())
                        .rtnWhsNm(m.getRtnWhsNm() == null ? "" : m.getRtnWhsNm() + " - " + m.getRtnLocationNm())
                        .rtnReqDt(m.getRtnReqDt() == null ? "" : m.getRtnReqDt())
                        .rtnReqItemQty(m.getRtnReqItemQty() == null ? 0 : m.getRtnReqItemQty())
                        .rtnReqAmt(m.getRtnReqAmt() == null ? 0 : m.getRtnReqAmt())
                        .colReqDt(m.getColReqDt() == null ? "" : m.getColReqDt())
                        .rtnReqMgrId(m.getRtnReqMgrId() == null ? "" : m.getRtnReqMgrId())
                        .rtnReqMgrNm(m.getRtnReqMgrNm() == null ? "" : m.getRtnReqMgrNm())
                        .rtnRjctDt(m.getRtnRjctDt() == null ? "" : m.getRtnRjctDt())
                        .rtnRjctNm(m.getRtnRjctNm() == null ? "" : m.getRtnRjctNm())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }


    /* 반품요청 정보 */
    public RtnReqInfoResponse selectRtnReqInfo(String userId, String brandId, String  rtnReqUnqNo) throws Exception {

        String storeId = "";

        // 로그인자 권한이 50 이면 매장 아이디 필수로 넘어와야함
        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);

        if ("50".equals(uEntity.getAuthTpCd())) {

            if (!brandId.equals(uEntity.getBrandId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
            }

            storeId = uEntity.getStoreId();

        } else if ("40".equals(uEntity.getAuthTpCd())){
            if (!brandId.equals(uEntity.getBrandId())) {
                throw new GlobalException(SystemMessageCode.ERR_NOTBRANDBELONG);
            }
        }

        RtnReqBaseInfoEntity entity = rtnReqMapper.selectRtnReqInfo(brandId, storeId,  rtnReqUnqNo);
        List<RtnReqDtlPtclEntity> dtlEntity = rtnReqMapper.selectRtnReqDtlList(rtnReqUnqNo);



        return RtnReqInfoResponse.toDTO(entity, dtlEntity);
    }
}
